@@ -11,25 +11,24 @@ import json
 from collections import namedtuple
 import numpy as np
 from scipy.interpolate import UnivariateSpline
+from packaging.version import parse as version_parse
 
-from sqlalchemy import MetaData, create_engine
+import sqlalchemy
 from sqlalchemy.orm import sessionmaker
-from sqlalchemy.pool import SingletonThreadPool
 
 from .utils import elam_spline, as_ndarray
 
 XrayEdge = namedtuple('XrayEdge', ('energy', 'fyield', 'jump_ratio'))
 XrayLine = namedtuple('XrayLine', ('energy', 'intensity', 'initial_level',
                                    'final_level'))
-ElementData = namedtuple('ElementData', ('Z', 'symbol', 'mass', 'density'))
+ElementData = namedtuple('ElementData', ('Z', 'symbol', 'name', 'mass', 'density'))
+ComptonEnergies = namedtuple('ComptonEnergies', ('incident', 'xray_90deg', 'xray_mean', 'electron_mean'))
 
-__version__ = '1.4'
+__version__ = '1.5'
 
 def make_engine(dbname):
-    "create engine for sqlite connection"
-    return create_engine('sqlite:///%s' % (dbname),
-                         poolclass=SingletonThreadPool,
-                         connect_args={'check_same_thread': False})
+    "create engine for sqlite connection, perhaps trying a few sqlachemy variants"
+    return sqlalchemy.create_engine(f'sqlite:///{dbname}')
 
 def isxrayDB(dbname):
     """whether a file is a valid XrayDB database
@@ -46,13 +45,13 @@ def isxrayDB(dbname):
         'Chantler', 'Waasmaier', and 'KeskiRahkonen_Krause'
     """
     _tables = ('Chantler', 'Waasmaier', 'Coster_Kronig',
-               'KeskiRahkonen_Krause', 'xray_levels',
-               'elements', 'photoabsorption', 'scattering')
+               'KeskiRahkonen_Krause', 'xray_levels', 'elements',
+               'photoabsorption', 'scattering')
     result = False
     try:
         engine = make_engine(dbname)
-        meta = MetaData(engine)
-        meta.reflect()
+        meta = sqlalchemy.MetaData()
+        meta.reflect(engine)
         result = all([t in meta.tables for t in _tables])
     except:
         pass
@@ -82,8 +81,8 @@ class XrayDB():
 
         if not isxrayDB(dbname):
             raise ValueError("'%s' is not a valid X-ray Database file!" % dbname)
-
-        self.dbname = dbname
+        self._cache = {}
+        self.dbname = os.path.abspath(dbname)
         self.engine = make_engine(dbname)
         self.conn = self.engine.connect()
         kwargs = {}
@@ -93,16 +92,18 @@ class XrayDB():
             def readonly_flush(*args, **kwargs):
                 return
 
-            self.session = sessionmaker(bind=self.engine, **kwargs)()
+            self.session = sessionmaker(self.engine, **kwargs)()
             self.session.flush = readonly_flush
         else:
             self.session = sessionmaker(bind=self.engine, **kwargs)()
 
-        self.metadata = MetaData(self.engine)
-        self.metadata.reflect()
+        self.metadata = sqlalchemy.MetaData()
+        self.metadata.reflect(self.engine)
         self.tables = self.metadata.tables
-        elems = self.tables['elements'].select().execute()
-        self.atomic_symbols = [e.element for e in elems.fetchall()]
+
+        elems = self.get_cache('elements')
+        self.__atomic_symbols = [e.element for e in elems]
+        self.__atomic_names = [e.name for e in elems]
 
     def close(self):
         "close session"
@@ -113,6 +114,35 @@ class XrayDB():
         "generic query"
         return self.session.query(*args, **kws)
 
+    def get_cache(self, tablename, column=None, value=None):
+        """for some tables, we will just cache all the data"""
+        if column is None:
+            if tablename in self._cache:
+                rows = self._cache[tablename]
+            else:
+                q = self.tables[tablename].select()
+                rows = self.session.execute(q).fetchall()
+                self._cache[tablename] = rows
+                # print("cached ", tablename)
+        else:
+            if tablename in self._cache:
+                data = self._cache[tablename]
+            else:
+                data = self._cache[tablename] = {}
+
+            key = f"{column:s}_{repr(value)}"
+            if key not in data:
+                tab = self.tables[tablename]
+                col = getattr(tab.c, column, None)
+                if col is None:
+                    raise ValueError(f"now column {column} for table {tablename}")
+                data[key] = self.query(tab).filter(col==value).all()
+                # print("cached ", tablename, key)
+            if key in data:
+                rows = data[key]
+        return rows
+
+
     def get_version(self, long=False, with_history=False):
         """
         return sqlite3 database and python library version numbers
@@ -125,7 +155,7 @@ class XrayDB():
             string: version information
         """
         out = []
-        rows = self.tables['Version'].select().execute().fetchall()
+        rows = self.get_cache('Version')
         if not with_history:
             rows = rows[-1:]
         if long or with_history:
@@ -164,12 +194,10 @@ class XrayDB():
         References:
             Waasmaier and Kirfel
         """
-        wtab = self.tables['Waasmaier']
-        rows = self.query(wtab)
+        rows = self.get_cache('Waasmaier')
         if element is not None:
-            elem = self.symbol(element)
-            rows = rows.filter(wtab.c.element == elem)
-        return [str(r.ion) for r in rows.all()]
+            rows = [r for r in rows if r.element==element]
+        return [str(r.ion) for r in rows]
 
     def f0(self, ion, q):
         """
@@ -201,13 +229,17 @@ class XrayDB():
         References:
             Waasmaier and Kirfel
         """
-        wtab = self.tables['Waasmaier']
+        f0_rows = self.get_cache('Waasmaier')
+        row = [None]
         if isinstance(ion, int):
-            row = self.query(wtab).filter(wtab.c.atomic_number == ion).all()[0]
-        elif ion not in self.f0_ions():
-            raise ValueError('No ion {:s} from Waasmaier table'.format(repr(ion)))
+            rows = [r for r in f0_rows if ion == r.atomic_number]
+            if len(rows) == 0:
+                raise ValueError(f'No ion {ion} from Waasmaier table')
         else:
-            row = self.query(wtab).filter(wtab.c.ion == ion.title()).all()[0]
+            rows = [r for r in f0_rows if ion.title() == r.ion]
+            if len(rows) == 0:
+                raise ValueError(f'No ion {ion} from Waasmaier table')
+        row = rows[0]
         q = as_ndarray(q)
         f0 = row.offset
         for s, e in zip(json.loads(row.scale), json.loads(row.exponents)):
@@ -226,13 +258,12 @@ class XrayDB():
         Notes:
            this function is meant for internal use.
         """
-        ctab = self.tables['Chantler']
         elem = self.symbol(element)
-        row = self.query(ctab).filter(ctab.c.element == elem).one()
-
         energy = as_ndarray(energy)
         emin, emax = min(energy), max(energy)
 
+        row = self.get_cache('Chantler', column='element', value=elem)[0]
+
         te = np.array(json.loads(row.energy))
         nemin = max(0, -3 + max(np.where(te <= emin)[0]))
         nemax = min(len(te), 3 + max(np.where(te <= emax)[0]))
@@ -346,19 +377,37 @@ class XrayDB():
             col = 'mu_incoh'
         return self._from_chantler(element, energy, column=col)
 
+    def compton_energies(self, incident_energy):
+        """
+        return tuple of Compton energies for an incident energy
+        """
+        row = self.get_cache('Compton_energies')[0]
+        _en   = json.loads(row.incident)
+        _xe90 = json.loads(row.xray_90deg)
+        _xave = json.loads(row.xray_mean)
+        _eave = json.loads(row.electron_mean)
+
+        xray_90deg = np.interp(incident_energy, _en, _xe90)
+        xray_mean = np.interp(incident_energy, _en, _xave)
+        electron_mean = np.interp(incident_energy, _en, _eave)
+
+        return ComptonEnergies(incident_energy, xray_90deg, xray_mean, electron_mean)
+
     def _elem_data(self, element):
         "return data from elements table: internal use"
-        etab = self.tables['elements']
-        row = self.query(etab)
+
+        rows = self.get_cache('elements')
         if isinstance(element, int):
-            row = row.filter(etab.c.atomic_number == element).one()
+            row = [r for r in rows if r.atomic_number == element][0]
         else:
-            elem = element.title()
-            if not elem in self.atomic_symbols:
-                raise ValueError("unknown element '%s'" % repr(elem))
-            row = row.filter(etab.c.element == elem).one()
+            if element.title() in self.__atomic_symbols:
+                row = [r for r in rows if r.element == element.title()][0]
+            elif element.lower() in self.__atomic_names:
+                row = [r for r in rows if r.name == element.lower()][0]
+            else:
+                raise ValueError("unknown element '%s'" % repr(element))
         return ElementData(int(row.atomic_number),
-                           row.element.title(),
+                           row.element.title(), row.name,
                            row.molar_mass, row.density)
 
     def atomic_number(self, element):
@@ -385,6 +434,18 @@ class XrayDB():
         """
         return self._elem_data(element).symbol
 
+    def atomic_name(self, element):
+        """
+        return element's name (English)
+
+        Parameters:
+            element (string or int): atomic number or symbol
+
+        Returns:
+            string: element name
+        """
+        return self._elem_data(element).name
+
     def molar_mass(self, element):
         """
         return molar mass of element
@@ -419,7 +480,7 @@ class XrayDB():
 
         Returns:
             dictionary:  keys of edge (iupac symbol), and values of
-                         XrayEdge namedtuple of (energy, fyield, edge_jump))
+                         XrayEdge namedtuple of (energy, fyield, edge_jump)
 
         References:
            Elam, Ravel, and Sieber.
@@ -442,7 +503,7 @@ class XrayDB():
             edge (string):  X-ray edge
 
         Returns:
-            XrayEdge:  namedtuple of (energy, fyield, edge_jump))
+            XrayEdge:  namedtuple of (energy, fyield, edge_jump)
 
         Example:
             >>> xdb = XrayDB()
@@ -596,7 +657,7 @@ class XrayDB():
 
         """
         version_qy = self.tables['Version'].select().order_by('date')
-        version_id = version_qy.execute().fetchall()[-1].id
+        version_id = self.session.execute(version_qy).fetchall()[-1].id
 
         ctab = self.tables['corelevel_widths']
         if version_id < 4 or use_keski:
@@ -635,11 +696,9 @@ class XrayDB():
         if kind not in ('coh', 'incoh', 'photo'):
             raise ValueError('unknown cross section kind=%s' % kind)
 
-        stab = self.tables['scattering']
-        if kind == 'photo':
-            stab = self. tables['photoabsorption']
+        tablename = 'photoabsorption' if kind == 'photo' else 'scattering'
 
-        row = self.query(stab).filter(stab.c.element == elem).all()[0]
+        row = self.get_cache(tablename, column='element', value=elem)[0]
 
         tab_lne = np.array(json.loads(row.log_energy))
         if kind.startswith('coh'):
@@ -656,8 +715,10 @@ class XrayDB():
         emin_tab = 10*int(0.102*np.exp(tab_lne[0]))
         en[np.where(en < emin_tab)] = emin_tab
         out = np.exp(elam_spline(tab_lne, tab_val, tab_spl, np.log(en)))
-        if len(out) == 1:
+
+        if isinstance(energies, (int, float)):
             return out[0]
+
         return out
 
     def mu_elam(self, element, energies, kind='total'):
